from utsam.dbs.schemas.masterdata.assignments import PgAssignmentsTable
from utsam.dbs.schemas.masterdata.courses import PgCoursesTable
from utsam.dbs.schemas.masterdata.criterias import PgAssignmentCriteriasTable
from utsam.dbs.schemas.masterdata.employees import PgEmployeesTable
from utsam.dbs.schemas.masterdata.grades import PgGradesTable
from utsam.dbs.schemas.masterdata.semesters import PgSemestersTable
from utsam.dbs.schemas.masterdata.sessions import PgSessionsTable
from utsam.dbs.schemas.masterdata.students import PgStudentsTable
from utsam.dbs.schemas.masterdata.subcriterias import PgAssignmentSubcriteriasTable
